% Joint Kernel learning via semi-infinite linear programming (SILP) in multi-class case
% Coded by Jianhui Chen (jianhui.chen@asu.edu, Arizona State University)

%                Input Parameters
% "G_Ker" : the cell containing centered kernel matrix (trace value = 1) 
% "H" : the indicator matrix
% "epislon" : tolerance parameter


%               Output parameters
% "theta0" : the coefficients of kernel matrices
% "lambda" : regularization parameter
% "time" : computation time in seconds

function  [ theta0 lambda time ] = SILP_multi_joint( G_Ker, H, epislon )

time0 = cputime;

[ n, K ] = size( H );
len = length(G_Ker) + 1;
G{1} = diag( ones(1,n)/n ) ;
G(2 : len) =G_Ker; 

zeta = 10^300 ; 
theta = ones(1, len) / len;

f = [ ]; A = [ ]; b = [ ] ; B = [ ]; c = [ ]; l = [ ] ; u = [ ];
[ f A b B c l u ] = lp_con( len );

% alpha_set = [ ];

for t = 1 : 1000
    
    %t0 = cputime;
    [ alpha val ] = compute_alpha_ls_joint(G, theta,H);
    %t1 = cputime; silp_time = t1- t0
        
    if (val > zeta) | ( abs(1-val/zeta) < epislon )
        break;
    else 
      %  ind = length(alpha_set)+1;
       % alpha_set{ind} = alpha; 
        
        %t0 = cputime;
        [ theta, zeta, f, A, b, B, c, l, u ] = compute_theta_lpprog_joint( G, alpha, H, f, A, b, B, c, l, u );
        %t1 = cputime;  time_lpprog(t) = t1- t0
    end
    
    %   t0 = cputime;
    %   [ theta2, zeta2 ] = compute_theta_general( G, alpha_set, lambda, H );
    %   t1 = cputime;   time_mosek(t) = t1- t0
%     zeta
%     val
end

lambda = theta(1);
theta0 = theta(2:end);
time = cputime - time0 ;